Volumetric = LibStub("AceAddon-3.0"):NewAddon("Volumetric", "AceConsole-3.0", "AceTimer-3.0")
local media = LibStub("LibSharedMedia-3.0")
media:Register("border", "Volumetric Flat", "Interface\\AddOns\\Volumetric\\flat.blp")

local defaults = {
	global = {
		texture = "Blizzard Raid Bar",
		bgTexture = "Blizzard Raid Bar",
		border = "Volumetric Flat",
		color = {0,0.478,1,1},
		bgColor = {0,0,0,.68},
		borderColor = {0,0,0,1},
		offy = -64,
		borderSize = 15,
		borderOffset = 10,
		alpha = 1
	}
}
local optionsTable = {
	name = "Volumetric |cff888888v" .. GetAddOnMetadata("Volumetric", "Version") .. "|r",
	type = "group",
	
	get = function(info)
		if info.type == "color" then
			local color = Volumetric.db.global[info[1]]
			return color[1], color[2], color[3], color[4]
		else
			return Volumetric.db.global[info[1]]
		end
	end,
	
	set = function(info, ...)
		local values = {...}
		
		if info.type == "color" then
			Volumetric.db.global[info[1]] = {values[1],values[2],values[3],values[4]}
		else
			Volumetric.db.global[info[1]] = values[1]
		end
		
		Volumetric:Refresh()
	end,
	
	args = {
		texture = {
			type = "select",
			dialogControl = "LSM30_Statusbar",
			name = "Bar Texture",
			values = AceGUIWidgetLSMlists.statusbar,
			order = 1,
			width = "double",
		},
		
		color = {
			type = "color",
			name = "Color",
			hasAlpha = false,
			order = 2,
		},
		
		desc1={type="description",name="",order=3},
		
		bgTexture = {
			type = "select",
			dialogControl = "LSM30_Statusbar",
			name = "Background Texture",
			values = AceGUIWidgetLSMlists.statusbar,
			order = 4,
			width = "double",
		},
		
		bgColor = {
			type = "color",
			name = "Color",
			hasAlpha = true,
			order = 5,
		},
		
		desc2={type="description",name="",order=6},
		
		alpha = {
			name = "Opacity",
			type = "range",
			order = 7,
			min = 0,
			max = 1,
			step = 0.01,
			isPercent = true,
		},
		
		offy = {
			name = "Vertical Position",
			type = "range",
			order = 8,
			softMin = -100,
			softMax = 100,
			step = 1,
		},
		
		header1={type="header",name="Border",order=9},
		
		border = {
			type = "select",
			dialogControl = "LSM30_Border",
			name = "Border",
			values = AceGUIWidgetLSMlists.border,
			order = 10,
			width="double"
		},
		
		borderColor = {
			type = "color",
			name = "Color",
			hasAlpha = false,
			order = 11,
		},
		
		borderOffset = {
			type = "range",
			name = "Offset",
			softMin = 0,
			softMax = 30,
			step = 1,
			order = 12,
			
		},
		
		borderSize = {
			type = "range",
			name = "Thickness",
			softMin = 1,
			softMax = 50,
			step = 1,
			order = 13,
			
		},
	}
}

function Volumetric:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("VolumetricDB", defaults, true) -- Initialize database
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Volumetric", optionsTable) -- Initialize options table
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Volumetric") -- Add options frame to Interface settings
	self.optionsFrame.default = self.ResetToDefaults
	media.RegisterCallback(self, "LibSharedMedia_Registered", "MediaUpdate")
	self:RegisterChatCommand("vm", "ChatCommand")
	self:RegisterChatCommand("vol", "ChatCommand")
	self:RegisterChatCommand("volumetric", "ChatCommand")
	
	self.optionsFrame:HookScript("OnShow", function()
		Volumetric.bar:SetFrameStrata("MEDIUM")
		Volumetric.bar:SetAlpha(self.db.global.alpha)
		Volumetric.bar:Show()
	end)
	
	self.optionsFrame:HookScript("OnHide", function()
		Volumetric.bar.fadeOut = true
		Volumetric.bar:SetFrameStrata("TOOLTIP")
		Volumetric.bar:SetAlpha(0)
		Volumetric.bar:Hide()
	end)
	
	self:InitializeFrame()
end

function Volumetric:ResetToDefaults()
	for k,v in pairs(defaults.global) do
		Volumetric.db.global[k] = v
	end
	
	LibStub("AceConfigRegistry-3.0"):NotifyChange("Volumetric")
	Volumetric:Refresh()
end
	
function Volumetric:StartFadingOut()
	self.bar.fadeOut = true
end

function Volumetric:InitializeFrame()
	self.bar = CreateFrame("StatusBar", "Volumetric", UIParent)
	self.bar:SetStatusBarTexture(media:Fetch("statusbar", self.db.global.texture))
	self.bar:GetStatusBarTexture():SetHorizTile(false)
	self.bar:SetMinMaxValues(0,1)
	self.bar:SetValue(GetCVar("Sound_MasterVolume"))
	self.bar:SetSize(250, 20)
	self.bar:SetFrameStrata("TOOLTIP")
	self.bar.fadeOut = false
	self.bar:Hide()
	self.bar:SetScript("OnUpdate", self.Update)
	
	-- Check UIParent:GetScale() on PLAYER_ENTERING_WORLD when its scale is correctly set
	self.bar:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.bar:SetScript("OnEvent", function()
		hooksecurefunc("SetCVar", function(cvar,value)
			if cvar == "Sound_MasterVolume" then
				self.bar:SetValue(value)
				self.bar.fadeOut = false
				self.bar:SetAlpha(self.db.global.alpha)
				self.bar:Show()
				self:CancelTimer(self.timer)
				self.timer = self:ScheduleTimer("StartFadingOut", 1)
			end
		end)
		
		self.bar:SetPoint("CENTER", 0, self.db.global.offy*3.84/UIParent:GetScale())
	end)
	
	self.bar.border = CreateFrame("Frame", "VolumetricBorder", self.bar)
	self.bar.border:SetPoint("CENTER")
	self.bar.border:SetBackdropColor(0,0,0,0)
	self.bar.bg = self.bar:CreateTexture(nil,"BACKGROUND")
	self.bar.bg:SetAllPoints()
	
	self:Refresh()
end

function Volumetric:Refresh()
	self.bar:SetStatusBarTexture(media:Fetch("statusbar", self.db.global.texture))
	self.bar:SetPoint("CENTER", 0, self.db.global.offy*3.84/UIParent:GetScale())
	self.bar:SetStatusBarColor(self.db.global.color[1],self.db.global.color[2],self.db.global.color[3],1)
	
	self.bar.bg:SetTexture(media:Fetch("statusbar", self.db.global.bgTexture))
	self.bar.bg:SetVertexColor(self.db.global.bgColor[1],self.db.global.bgColor[2],self.db.global.bgColor[3])
	self.bar.bg:SetAlpha(self.db.global.bgColor[4])
	
	self.bar.border:SetSize(self.bar:GetWidth() + self.db.global.borderOffset, self.bar:GetHeight() + self.db.global.borderOffset)
	self.bar.border:SetBackdrop({
		bgFile = nil,
		edgeFile = media:Fetch("border", self.db.global.border),
		tile=true, tileSize=5, edgeSize=self.db.global.borderSize
	})
	self.bar.border:SetBackdropBorderColor(self.db.global.borderColor[1],self.db.global.borderColor[2],self.db.global.borderColor[3],self.db.global.borderColor[4])
end

function Volumetric:Update(delta)
	if Volumetric.optionsFrame:IsVisible() then return end -- Always show when options are open
	
	if self.fadeOut then
		self:SetAlpha(self:GetAlpha() - delta * 2)
	end
	
	if self:GetAlpha() <= 0 then
		self:Hide()
	end
end

function Volumetric:MediaUpdate(event, mediatype, key)
	if (mediatype == "statusbar" and key == self.db.global.texture)or
	(mediatype == "statusbar" and key == self.db.global.bgTexture) or
	(mediatype == "border" and key == self.db.global.border) then
		Volumetric:Refresh()
	end
end

function Volumetric:ChatCommand(input)
	args = {strsplit(" ", input:trim())}
	
	if args[1] == "v" or args[1] == "ver" or args[1] == "version" then
		self:Print(GAME_VERSION_LABEL..": |cffffd700"..GetAddOnMetadata("Volumetric","Version").."|r")
	else
		InterfaceOptionsFrame_OpenToCategory("Volumetric")
		InterfaceOptionsFrame_OpenToCategory("Volumetric")
	end
end

function Volumetric:Print(message,msgType)
	DEFAULT_CHAT_FRAME:AddMessage("|cffd2b48c[Volumetric]|r "..message.."|r")
end
